@@ -1,10 +1,4 @@
 #!/usr/bin/env python
-#encoding:utf-8
-#author:dbr/Ben
-#project:tvnamer
-#repository:http://github.com/dbr/tvnamer
-#license:Creative Commons GNU GPL v2
-# http://creativecommons.org/licenses/GPL/2.0/
 
 """Holds default config values
 """
@@ -34,6 +28,11 @@ defaults = {
     # No leading dot, for example: ['avi', 'mkv', 'mp4']
     'valid_extensions': [],
 
+    # When non-empty, filter out filenames that match these expressions. Either simple
+    # matches or regexs can be used. The following are near enough equivalent:
+    # [{"is_regex": true, "match": ".*sample.*"}, {"is_regex": false, "match": "sample"}]
+    'filename_blacklist': [],
+
     # Force Windows safe filenames (always True on Windows)
     'windows_safe_filenames': False,
 
@@ -77,6 +76,13 @@ defaults = {
     # If False, will move files when renaming.
     'move_files_confirmation': True,
 
+    # If true, convert the variable/dynamic parts of the destination
+    # to lower case. Does not affect the static parts; for example,
+    # if move_files_destination is set to
+    # '/Foo/Bar/%(seriesname)s/Season %(seasonnumber)d'
+    # then only the series name will be converted to lower case.
+    'move_files_lowercase_destination': False,
+
     # Destination to move files to. Trailing slash is not necessary.
     # Use forward slashes, even on Windows. Realtive paths are realtive to
     # the existing file's path (not current working dir). A value of '.' will
@@ -175,11 +181,11 @@ defaults = {
         (?P<seasonnumber>[0-9]+)                 # 1
         [xX](?P<episodenumberstart>[0-9]+)       # first x23
         (                                        # -24 etc
-             [\-][0-9]+
+             [\-+][0-9]+
         )*
-             [\-]                                # separator
+             [\-+]                               # separator
              (?P<episodenumberend>[0-9]+)        # final episode num
-        ([\.\- ].*                               # must have a separator (prevents 1x01-720p from being 720 episodes)
+        ([\.\-+ ].*                              # must have a separator (prevents 1x01-720p from being 720 episodes)
         |
         $)''',
 
@@ -189,12 +195,18 @@ defaults = {
             ?(?P<seasonnumber>[0-9]+)            # season
         [xX]                                     # x
             (?P<episodenumberstart>[0-9]+)       # episode
-            (- [0-9]+)*
-        -                                        # -
+            ([\-+] [0-9]+)*
+        [\-+]                                    # -
             (?P<episodenumberend>[0-9]+)         # episode
         \]                                       # \]
         [^\\/]*$''',
 
+        # foo - [012]
+        '''^((?P<seriesname>.+?)[ \._\-])?       # show name and padding
+        \[                                       # [ not optional (or too ambigious)
+        (?P<episodenumber>[0-9]+)                # episode
+        \]                                       # ]
+        [^\\/]*$''',
         # foo.s0101, foo.0201
         '''^(?P<seriesname>.+?)[ \._\-]
         [Ss](?P<seasonnumber>[0-9]{2})
@@ -229,6 +241,17 @@ defaults = {
         (?P<day>\d{2})                           # day
         [^\/]*$''',
 
+        # foo - [01.09]
+        '''^((?P<seriesname>.+?))                # show name
+        [ \._\-]?                                # padding
+        \[                                       # [
+        (?P<seasonnumber>[0-9]+?)                # season
+        [.]                                      # .
+        (?P<episodenumber>[0-9]+?)               # episode
+        \]                                       # ]
+        [ \._\-]?                                # padding
+        [^\\/]*$''',
+
         # Foo - S2 E 02 - etc
         '''^(?P<seriesname>.+?)[ ]?[ \._\-][ ]?
         [Ss](?P<seasonnumber>[0-9]+)[\.\- ]?
@@ -277,7 +300,6 @@ defaults = {
         [Ee](?P<episodenumber>[0-9]+)            # E123
         [\._ -][^\\/]*$                          # More padding, then anything
         ''',
-
     ],
 
     # Formats for renamed files. Variations for with/without episode,
@@ -306,4 +328,10 @@ defaults = {
 
     # String to join multiple number
     'episode_separator': '-',
+    
+    # Series ID to use instead of searching if the value is set
+    #'series_id': None,
+    
+    # Forced Name to use
+    #'forced_name': None,
 }
