@@ -1,15 +1,10 @@
 #!/usr/bin/env python
-#encoding:utf-8
-#author:dbr/Ben
-#project:tvnamer
-#repository:http://github.com/dbr/tvnamer
-#license:Creative Commons GNU GPL v2
-# http://creativecommons.org/licenses/GPL/2.0/
 
 """Main tvnamer utility functionality
 """
 
 import os
+import sys
 import logging
 
 try:
@@ -17,7 +12,11 @@ try:
 except ImportError:
     pass
 
-import simplejson as json
+try:
+    import json
+except ImportError:
+    import simplejson as json
+
 from tvdb_api import Tvdb
 
 import cliarg_parser
@@ -25,8 +24,7 @@ from config_defaults import defaults
 
 from unicode_helper import p
 from utils import (Config, FileFinder, FileParser, Renamer, warn,
-getEpisodeName, applyCustomInputReplacements, applyCustomOutputReplacements,
-formatEpisodeNumbers, makeValidFilename)
+applyCustomInputReplacements, formatEpisodeNumbers, makeValidFilename)
 
 from tvnamer_exceptions import (ShowNotFound, SeasonNotFound, EpisodeNotFound,
 EpisodeNameNotFound, UserAbort, InvalidPath, NoValidFilesFoundError,
@@ -43,11 +41,26 @@ def getDestinationFolder(episode):
     """Constructs the location to move/copy the file
     """
 
+    #TODO: Write functional test to ensure this valid'ifying works
+    def wrap_validfname(fname):
+        """Wrap the makeValidFilename function as it's called twice
+        and this is slightly long..
+        """
+        if Config['move_files_lowercase_destination']:
+            fname = fname.lower()
+        return makeValidFilename(
+            fname,
+            normalize_unicode = Config['normalize_unicode_filenames'],
+            windows_safe = Config['windows_safe_filenames'],
+            custom_blacklist = Config['custom_filename_character_blacklist'],
+            replace_with = Config['replace_invalid_characters_with'])
+
+
     # Calls makeValidFilename on series name, as it must valid for a filename
     destdir = Config['move_files_destination'] % {
-        'seriesname': makeValidFilename(episode.seriesname),
+        'seriesname': wrap_validfname(episode.seriesname),
         'seasonnumber': episode.seasonnumber,
-        'episodenumbers': makeValidFilename(formatEpisodeNumbers(episode.episodenumbers))
+        'episodenumbers': wrap_validfname(formatEpisodeNumbers(episode.episodenumbers))
     }
     return destdir
 
@@ -59,7 +72,7 @@ def doRenameFile(cnamer, newName):
     try:
         cnamer.newName(newName)
     except OSError, e:
-        warn(unicode(e))
+        warn(e)
 
 
 def doMoveFile(cnamer, destDir, getPathPreview = False):
@@ -73,7 +86,7 @@ def doMoveFile(cnamer, destDir, getPathPreview = False):
     try:
         return cnamer.newPath(destDir, getPathPreview = getPathPreview)
     except OSError, e:
-        warn(unicode(e))
+        warn(e)
 
 
 def confirm(question, options, default = "y"):
@@ -118,7 +131,7 @@ def processFile(tvdb_instance, episode):
     p("# Detected series: %s (%s)" % (episode.seriesname, episode.number_string()))
 
     try:
-        correctedSeriesName, epName = getEpisodeName(tvdb_instance, episode)
+        episode.populateFromTvdb(tvdb_instance, force_name=Config['force_name'], series_id=Config['series_id'])
     except (DataRetrievalError, ShowNotFound), errormsg:
         if Config['always_rename'] and Config['skip_file_on_error'] is True:
             warn("Skipping file due to error: %s" % errormsg)
@@ -127,15 +140,11 @@ def processFile(tvdb_instance, episode):
             warn(errormsg)
     except (SeasonNotFound, EpisodeNotFound, EpisodeNameNotFound), errormsg:
         # Show was found, so use corrected series name
-        if Config['always_rename'] and Config['skip_file_on_error'] is True:
+        if Config['always_rename'] and Config['skip_file_on_error']:
             warn("Skipping file due to error: %s" % errormsg)
             return
 
         warn(errormsg)
-        episode.seriesname = correctedSeriesName
-    else:
-        episode.seriesname = correctedSeriesName
-        episode.episodename = epName
 
     cnamer = Renamer(episode.fullpath)
     newName = episode.generateFilename()
@@ -148,12 +157,9 @@ def processFile(tvdb_instance, episode):
         p("#" * 20)
         p("Old filename: %s" % episode.fullfilename)
 
-        if len(Config['output_filename_replacements']):
-            p("Before custom output replacements: %s" % (newName))
-            # Only apply to filename, not extension
-            newName, newExt = os.path.splitext(newName)
-            newName = applyCustomOutputReplacements(newName)
-            newName = newName + newExt
+        if len(Config['output_filename_replacements']) > 0:
+            # Show filename without replacements
+            p("Before custom output replacements: %s" % (episode.generateFilename(preview_orig_filename = False)))
 
         p("New filename: %s" % newName)
 
@@ -210,6 +216,7 @@ def findFiles(paths):
         cur = FileFinder(
             cfile,
             with_extension = Config['valid_extensions'],
+            filename_blacklist = Config["filename_blacklist"],
             recursive = Config['recursive'])
 
         try:
@@ -237,6 +244,11 @@ def tvnamer(paths):
             "It has not been heavily tested, and is not recommended for "
             "general use yet.")
 
+    if Config['force_name'] is not None:
+        import warnings
+        warnings.warn("The --name argument is a temporary solution, and will"
+            "be removed at some point in the future. Do no depend on it")
+
     p("#" * 20)
     p("# Starting tvnamer")
 
@@ -246,10 +258,13 @@ def tvnamer(paths):
         parser = FileParser(cfile)
         try:
             episode = parser.parse()
-        except InvalidFilename:
-            warn("Invalid filename %s" % cfile)
+        except InvalidFilename, e:
+            warn("Invalid filename: %s" % e)
         else:
-            episodes_found.append(episode)
+            if episode.seriesname is None:
+                warn("Parsed filename did not contain series name, skipping: %s" % cfile)
+            else:
+                episodes_found.append(episode)
 
     if len(episodes_found) == 0:
         raise NoValidFilesFoundError()
@@ -260,7 +275,7 @@ def tvnamer(paths):
     episodes_found.sort(key = lambda x: x.sortable_info())
 
     tvdb_instance = Tvdb(
-        interactive=not Config['select_first'],
+        interactive = not Config['select_first'],
         search_all_languages = Config['search_all_languages'],
         language = Config['language'])
 
@@ -279,8 +294,6 @@ def main():
 
     opts, args = opter.parse_args()
 
-    args = [x.decode("utf-8") for x in args]
-
     if opts.verbose:
         logging.basicConfig(
             level = logging.DEBUG,
@@ -315,6 +328,10 @@ def main():
             opter = cliarg_parser.getCommandlineParser(defaults)
             opts, args = opter.parse_args()
 
+    # Decode args using filesystem encoding (done after config loading
+    # as the args are reparsed when the config is loaded)
+    args = [x.decode(sys.getfilesystemencoding()) for x in args]
+
     # Save config argument
     if opts.saveconfig is not None:
         p("Saving config: %s" % (opts.saveconfig))
