@@ -1,10 +1,4 @@
 #!/usr/bin/env python
-#encoding:utf-8
-#author:dbr/Ben
-#project:tvnamer
-#repository:http://github.com/dbr/tvnamer
-#license:Creative Commons GNU GPL v2
-# http://creativecommons.org/licenses/GPL/2.0/
 
 """Utilities for tvnamer, including filename parsing
 """
@@ -35,82 +29,11 @@ def log():
 
 
 def warn(text):
-    """Displays message to sys.stdout
+    """Displays message to sys.stderr
     """
     p(text, file = sys.stderr)
 
 
-def getEpisodeName(tvdb_instance, episode):
-    """Queries the tvdb_api.Tvdb instance for episode name and corrected
-    series name.
-    If series cannot be found, it will warn the user. If the episode is not
-    found, it will use the corrected show name and not set an episode name.
-    If the site is unreachable, it will warn the user. If the user aborts
-    it will catch tvdb_api's user abort error and raise tvnamer's
-    """
-    try:
-        show = tvdb_instance[episode.seriesname]
-    except tvdb_error, errormsg:
-        raise DataRetrievalError("Error contacting www.thetvdb.com: %s" % errormsg)
-    except tvdb_shownotfound:
-        # No such series found.
-        raise ShowNotFound("Show %s not found on www.thetvdb.com" % episode.seriesname)
-    except tvdb_userabort, error:
-        raise UserAbort(unicode(error))
-    else:
-        # Series was found, use corrected series name
-        correctedShowName = show['seriesname']
-
-    if isinstance(episode, DatedEpisodeInfo):
-        # Date-based episode
-        epnames = []
-        for cepno in episode.episodenumbers:
-            try:
-                sr = show.airedOn(cepno)
-                if len(sr) > 1:
-                    raise EpisodeNotFound(
-                        "Ambigious air date %s, there were %s episodes on that day" % (
-                        cepno, len(sr)))
-                epnames.append(sr[0]['episodename'])
-            except tvdb_episodenotfound:
-                raise EpisodeNotFound(
-                    "Episode that aired on %s could not be found" % (
-                    cepno))
-        return correctedShowName, epnames
-
-    if episode.seasonnumber is None:
-        # Series without concept of seasons have all episodes in season 1
-        seasonnumber = 1
-    else:
-        seasonnumber = episode.seasonnumber
-
-    epnames = []
-    for cepno in episode.episodenumbers:
-        try:
-            episodeinfo = show[seasonnumber][cepno]
-
-        except tvdb_seasonnotfound:
-            raise SeasonNotFound(
-                "Season %s of show %s could not be found" % (
-                episode.seasonnumber,
-                episode.seriesname))
-
-        except tvdb_episodenotfound:
-            raise EpisodeNotFound(
-                "Episode %s of show %s, season %s could not be found" % (
-                    cepno,
-                    episode.seriesname,
-                    episode.seasonnumber))
-
-        except tvdb_attributenotfound:
-            raise EpisodeNameNotFound(
-                "Could not find episode name for %s" % episode)
-        else:
-            epnames.append(episodeinfo['episodename'])
-
-    return correctedShowName, epnames
-
-
 def _applyReplacements(cfile, replacements):
     """Applies custom replacements.
 
@@ -166,6 +89,26 @@ def cleanRegexedSeriesName(seriesname):
     return seriesname.strip()
 
 
+def handleYear(year):
+    """Handle two-digit years with heuristic-ish guessing
+
+    Assumes 50-99 becomes 1950-1999, and 0-49 becomes 2000-2049
+
+    ..might need to rewrite this function in 2050, but that seems like
+    a reasonable limitation
+    """
+
+    year = int(year)
+
+    # No need to guess with 4-digit years
+    if year > 999:
+        return year
+
+    if year < 50:
+        return 2000 + year
+    else:
+        return 1900 + year
+
 class FileFinder(object):
     """Given a file, it will verify it exists. Given a folder it will descend
     one level into it and return a list of files, unless the recursive argument
@@ -174,21 +117,30 @@ class FileFinder(object):
     The with_extension argument is a list of valid extensions, without leading
     spaces. If an empty list (or None) is supplied, no extension checking is
     performed.
+
+    The filename_blacklist argument is a list of regexp strings to match against
+    the filename (minus the extension). If a match is found, the file is skipped
+    (e.g. for filtering out "sample" files). If [] or None is supplied, no
+    filtering is done
     """
 
-    def __init__(self, path, with_extension = None, recursive = False):
+    def __init__(self, path, with_extension = None, filename_blacklist = None, recursive = False):
         self.path = path
         if with_extension is None:
             self.with_extension = []
         else:
             self.with_extension = with_extension
+        if filename_blacklist is None:
+            self.with_blacklist = []
+        else:
+            self.with_blacklist = filename_blacklist
         self.recursive = recursive
 
     def findFiles(self):
         """Returns list of files found at path
         """
         if os.path.isfile(self.path):
-            if self._checkExtension(self.path):
+            if self._checkExtension(self.path) and not self._blacklistedFilename(self.path):
                 return [os.path.abspath(self.path)]
             else:
                 return []
@@ -198,6 +150,8 @@ class FileFinder(object):
             raise InvalidPath("%s is not a valid file/directory" % self.path)
 
     def _checkExtension(self, fname):
+        """Checks if the file extension is blacklisted in valid_extensions
+        """
         if len(self.with_extension) == 0:
             return True
 
@@ -209,6 +163,23 @@ class FileFinder(object):
         else:
             return False
 
+    def _blacklistedFilename(self, fname):
+        """Checks if the filename (excl. ext) matches filename_blacklist
+        """
+        if len(self.with_blacklist) == 0:
+            return False
+
+        fname, _ = os.path.splitext(fname)
+        for fblacklist in self.with_blacklist:
+            if "is_regex" in fblacklist and fblacklist["is_regex"]:
+                if re.match(fblacklist["match"], fname):
+                    return True
+            else:
+                if fname.find(fblacklist["match"]) != -1:
+                    return True
+        else:
+            return False
+
     def _findFilesInPath(self, startpath):
         """Finds files from startpath, could be called recursively
         """
@@ -218,12 +189,15 @@ class FileFinder(object):
             return allfiles
 
         for subf in os.listdir(unicode(startpath)):
-            if not self._checkExtension(subf):
-                continue
             newpath = os.path.join(startpath, subf)
             newpath = os.path.abspath(newpath)
             if os.path.isfile(newpath):
-                allfiles.append(newpath)
+                if not self._checkExtension(subf):
+                    continue
+                elif self._blacklistedFilename(subf):
+                    continue
+                else:
+                    allfiles.append(newpath)
             else:
                 if self.recursive:
                     allfiles.extend(self._findFilesInPath(newpath))
@@ -296,7 +270,9 @@ class FileParser(object):
                             "Date-based regex must contain groups 'year', 'month' and 'day'")
                     match.group('year')
 
-                    episodenumbers = [datetime.date(int(match.group('year')),
+                    year = handleYear(match.group('year'))
+
+                    episodenumbers = [datetime.date(year,
                                                     int(match.group('month')),
                                                     int(match.group('day')))]
 
@@ -338,7 +314,10 @@ class FileParser(object):
 
                 return episode
         else:
-            raise InvalidFilename(self.path)
+            emsg = "Cannot parse %r" % self.path
+            if len(Config['input_filename_replacements']) > 0:
+                emsg += " with replacements: %r" % filename
+            raise InvalidFilename(emsg)
 
 
 def formatEpisodeName(names, join_with):
@@ -487,9 +466,9 @@ class EpisodeInfo(object):
     """
 
     def __init__(self,
-        seriesname = None,
-        seasonnumber = None,
-        episodenumbers= None,
+        seriesname,
+        seasonnumber,
+        episodenumbers,
         episodename = None,
         filename = None):
 
@@ -529,7 +508,100 @@ class EpisodeInfo(object):
             self.seasonnumber,
             ", ".join([str(x) for x in self.episodenumbers]))
 
-    def generateFilename(self, lowercase = False):
+    def populateFromTvdb(self, tvdb_instance, force_name=None, series_id=None):
+        """Queries the tvdb_api.Tvdb instance for episode name and corrected
+        series name.
+        If series cannot be found, it will warn the user. If the episode is not
+        found, it will use the corrected show name and not set an episode name.
+        If the site is unreachable, it will warn the user. If the user aborts
+        it will catch tvdb_api's user abort error and raise tvnamer's
+        """
+        try:
+            if series_id is None:
+                show = tvdb_instance[force_name or self.seriesname]
+            else:
+                series_id = int(series_id)
+                tvdb_instance._getShowData(series_id, Config['language'])
+                show = tvdb_instance[series_id]
+        except tvdb_error, errormsg:
+            raise DataRetrievalError("Error contacting www.thetvdb.com: %s" % errormsg)
+        except tvdb_shownotfound:
+            # No such series found.
+            raise ShowNotFound("Show %s not found on www.thetvdb.com" % self.seriesname)
+        except tvdb_userabort, error:
+            raise UserAbort(unicode(error))
+        else:
+            # Series was found, use corrected series name
+            self.seriesname = show['seriesname']
+
+        if isinstance(self, DatedEpisodeInfo):
+            # Date-based episode
+            epnames = []
+            for cepno in self.episodenumbers:
+                try:
+                    sr = show.airedOn(cepno)
+                    if len(sr) > 1:
+                        raise EpisodeNotFound(
+                            "Ambigious air date %s, there were %s episodes on that day" % (
+                            cepno, len(sr)))
+                    epnames.append(sr[0]['episodename'])
+                except tvdb_episodenotfound:
+                    raise EpisodeNotFound(
+                        "Episode that aired on %s could not be found" % (
+                        cepno))
+            self.episodename = epnames
+            return
+
+        if not hasattr(self, "seasonnumber") or self.seasonnumber is None:
+            # Series without concept of seasons have all episodes in season 1
+            seasonnumber = 1
+        else:
+            seasonnumber = self.seasonnumber
+
+        epnames = []
+        for cepno in self.episodenumbers:
+            try:
+                episodeinfo = show[seasonnumber][cepno]
+
+            except tvdb_seasonnotfound:
+                raise SeasonNotFound(
+                    "Season %s of show %s could not be found" % (
+                    seasonnumber,
+                    self.seriesname))
+
+            except tvdb_episodenotfound:
+                # Try to search by absolute_number
+                sr = show.search(cepno, "absolute_number")
+                if len(sr) > 1:
+                    # For multiple results try and make sure there is a direct match
+                    unsure = True
+                    for e in sr:
+                        if int(e['absolute_number']) == cepno:
+                            epnames.append(e['episodename'])
+                            unsure = False
+                    # If unsure error out            
+                    if unsure:
+                        raise EpisodeNotFound(
+                            "No episode actually matches %s, found %s results instead" % (cepno, len(sr)))
+                elif len(sr) == 1:
+                    epnames.append(sr[0]['episodename'])
+                else:
+                    raise EpisodeNotFound(
+                        "Episode %s of show %s, season %s could not be found (also tried searching by absolute episode number)" % (
+                            cepno,
+                            self.seriesname,
+                            seasonnumber))
+
+            except tvdb_attributenotfound:
+                raise EpisodeNameNotFound(
+                    "Could not find episode name for %s" % cepno)
+            else:
+                epnames.append(episodeinfo['episodename'])
+
+        self.episodename = epnames
+
+
+    def generateFilename(self, lowercase = False, preview_orig_filename = False):
         """
         Uses the following config options:
         filename_with_episode # Filename when episode name is found
@@ -566,6 +638,16 @@ class EpisodeInfo(object):
         if lowercase or Config['lowercase_filename']:
             fname = fname.lower()
 
+        if preview_orig_filename:
+            # Return filename without custom replacements of filesystem-validness
+            return fname
+
+        if len(Config['output_filename_replacements']) > 0:
+            # Only apply replacements to filename, not extension
+            splitname, splitext = os.path.splitext(fname)
+            newname = applyCustomOutputReplacements(splitname)
+            fname = newname + splitext
+
         return makeValidFilename(
             fname,
             normalize_unicode = Config['normalize_unicode_filenames'],
@@ -574,15 +656,15 @@ class EpisodeInfo(object):
             replace_with = Config['replace_invalid_characters_with'])
 
     def __repr__(self):
-        return "<%s: %s>" % (
+        return u"<%s: %r>" % (
             self.__class__.__name__,
             self.generateFilename())
 
 
 class DatedEpisodeInfo(EpisodeInfo):
     def __init__(self,
-        seriesname = None,
-        episodenumbers= None,
+        seriesname,
+        episodenumbers,
         episodename = None,
         filename = None):
 
@@ -643,8 +725,8 @@ class DatedEpisodeInfo(EpisodeInfo):
 
 class NoSeasonEpisodeInfo(EpisodeInfo):
     def __init__(self,
-        seriesname = None,
-        episodenumbers= None,
+        seriesname,
+        episodenumbers,
         episodename = None,
         filename = None):
 
